﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CG2.Mathematics;

namespace CG2.Lighting
{
	public class SpotLight : PointLight
	{
		#region Properties

		public Vector4 Direction;
		public Double CutoffDeg = 45; // 0 .. 180
		public Double Exp = 3;

		#endregion

		#region Lighting

		public override Double GetIntensityAt(Vector4 point)
		{
			Double intensity = base.GetIntensityAt(point);

			Vector4 lightVector = (point - Origin).Normalized; // light vector
			Double devCos = Direction.Normalized * lightVector;
			if (devCos < 0) return 0;
			
			Double devAngleDeg = MathEx.RadToDeg(Math.Acos(devCos));
			if (devAngleDeg > CutoffDeg) return 0;

			Double ratio = devAngleDeg / CutoffDeg;
			Double decay = 1 - Math.Pow(ratio, Exp);

			return decay * intensity;
		}

		public override void SetLightRayAt(Vector4 point, Rendering.Ray ray)
		{
			base.SetLightRayAt(point, ray);
		}

		#endregion
	}
}
