﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CG2.Mathematics;

namespace CG2.Texturing
{
	public class PlaneMapping : TextureMapping
	{
		#region Properties

		public Vector4 Origin;
		public Vector4 U;
		public Vector4 V;

		protected Vector4 unitU, unitV, n;
		protected Double uu, vv, uv, k;

		#endregion


		#region Init

		public PlaneMapping(Vector4 origin, Vector4 u, Vector4 v)
		{
			Origin = origin;
			U = u;
			V = v;
			uu = u * u;
			vv = v * v;
			uv = u * v;
			k = 1 / (uv * uv - uu * vv);
			n = (u % v).Normalized;
			unitU = U.Normalized;
			unitV = V.Normalized;
		}

		#endregion


		#region Mapping

		public override void GetTangetSpace(Vector4 point, out Vector4 u, out Vector4 v, out Vector4 w)
		{
			// TODO: set u,v,w being base vectors for the tangent space
			u = Vector4.Zero;
			v = Vector4.Zero;
			w = Vector4.Zero;
		}

		public override Vector4 GetCoords(Vector4 point)
		{
			Double noo = n * (Origin - point);
			Vector4 w = point + noo * n - Origin;
			Double wu = w * U;
			Double wv = w * V;
			Double s = k * (uv * wv - vv * wu);
			Double t = k * (uv * wu - uu * wv);

			return Offset + new Vector4(s, t, -noo);
		}

		#endregion
	}
}
